#include "Cyton_SD.h"
#include "usart.h"	
#include <string.h>
#include "delay.h"       
#include "led.h"       
#include "key.h"
#include "Timer.h"
#include "ADS129x.h"
#include "adc.h"
#include "fattester.h"


#include "myQueue.h"


static int g_SdInitResult = 0;
static int g_SdIsBusy = 0;
static FIL g_stfile;
static u8 sd_path[] = "0:";
static uint8_t g_aucWriteHeadIndex = 0;

int CytonSD_Init()
{
	int iRet = -1;
    exfuns_init();
	iRet = mf_mount(sd_path, 1);
    if (iRet == 0) {
	    Main_printf("mount end %d Success\r\n", iRet);
	    mf_showfree("0:");
        g_SdInitResult = 1;
    } else {
        Main_printf("mount end %d failed\r\n", iRet);
    }

    mf_scan_files("0:");
	mf_showfree("0:");

    return iRet;
}

int CytonSD_Start()
{
    int iLoop = 0;
    char szFileName[64] = {0};
    char szAbFileName[64] = {0};
    int iRet = 0;
    char szTmp[16] = {0};

    g_aucWriteHeadIndex = 0;
    if (1 != g_SdInitResult) {
        Main_printf("SD not mount\r\n");
        return -1;
    }

    for (iLoop = 0; iLoop < 256; iLoop ++) {
		memset(szFileName, 0, sizeof(szFileName));
		sprintf(szFileName, "OPBI_%d.txt", iLoop);
		if (0 == mf_scan_fileExist("0:", szFileName)) {
			Main_printf("file is not exist, start creat %s\r\n", szFileName);
			sprintf(szAbFileName, "0:%s", szFileName);
			break;
		}
	}

    iRet = f_open(&g_stfile, szAbFileName, FA_CREATE_NEW | FA_WRITE);
	Main_printf("f_open %d\r\n", iRet);
    if (0 == iRet) {
        sprintf(szTmp, "%08X\n", CytonSys_GetCurrentms());
        CytonSD_Write("%STOP AT\n", strlen("%STOP AT\n"));
        CytonSD_Write(szTmp, strlen(szTmp));
    }
    
    g_SdIsBusy = 1;
	
	f_sync(&g_stfile);

    return iRet;
}

int CytonSD_Write(char *szBuf, int iLen)
{
    int iRet = -1;
    int iWriten = 0;

    if (1 != g_SdInitResult) {
        return 0;
    }

    //f_lseek(&file, 1024*1024);
    iRet = f_write(&g_stfile, szBuf, iLen, &iWriten);
	if (iRet != 0) {
		Main_printf("[%d] %d iWriten %d failed\r\n", iRet);
	}
    //f_sync(&g_stfile);

    return iWriten;
}

int CytonSD_WriteChannelData(char *szBuf, int iChannel, int iNum)
{
    char szTmp[512] = {0};
    int iLoop = 0;
    int iOneBufLen = 0;
    int iLoopJ = 0;

    if (1 != g_SdInitResult) {
        return 0;
    }

    iOneBufLen = iChannel * 7 + 3;
    for (;iLoop < iNum; iLoop++) {
        g_aucWriteHeadIndex ++;
        sprintf((szTmp + iOneBufLen * iLoop), "%02X,", g_aucWriteHeadIndex);
        for (iLoopJ = 0; iLoopJ < iChannel; iLoopJ ++) {
            sprintf((szTmp + iOneBufLen * iLoop + 3 + iLoopJ * 7), "%02X", *(szBuf + iLoop * iChannel * 3 + iLoopJ * 3));
            sprintf((szTmp + iOneBufLen * iLoop + 3 + iLoopJ * 7 + 2), "%02X", *(szBuf + iLoop * iChannel * 3 + iLoopJ * 3 +1));
            if (iLoopJ == (iChannel - 1)) {
                sprintf((szTmp + iOneBufLen * iLoop + 3 + iLoopJ * 7 + 4), "%02X\n", *(szBuf + iLoop * iChannel * 3 + iLoopJ * 3 + 2));
            } else {
                sprintf((szTmp + iOneBufLen * iLoop + 3 + iLoopJ * 7 + 4), "%02X,", *(szBuf + iLoop * iChannel * 3 + iLoopJ * 3 + 2));
            }
        }
    }

    //printf("szTmp %s ****\n",szTmp);
    CytonSD_Write(szTmp, strlen(szTmp));
    return 0;
}

int CytonSD_Stop()
{
    char szTmp[16] = {0};


    if (1 != g_SdInitResult) {
        return 0;
    }

    //sprintf(szTmp, "%08X\n", CytonSys_GetCurrentms());
    //CytonSD_Write("%STOP AT\n", strlen("%STOP AT\n"));
    //CytonSD_Write(szTmp, strlen(szTmp));

    CytonSD_Write("%Total time mS:\n", strlen("%Total time mS:\n"));
    CytonSD_Write(szTmp, strlen(szTmp));

    CytonSD_Write("%min Write time uS:\n", strlen("%min Write time uS:\n"));
    CytonSD_Write(szTmp, strlen(szTmp));

    CytonSD_Write("%max Write time uS:\n", strlen("%max Write time uS:\n"));
    CytonSD_Write(szTmp, strlen(szTmp));

    CytonSD_Write("%Over:\n", strlen("%Over:\n"));
    CytonSD_Write(szTmp, strlen(szTmp));

    CytonSD_Write("%block, uS\n", strlen("%block, uS\n"));
    CytonSD_Write(szTmp, strlen(szTmp));

    g_SdIsBusy = 0;
    g_aucWriteHeadIndex = 0;
    f_close(&g_stfile);
    return 0;
}

int CytonSD_GetStatus()
{
    return (g_SdInitResult + g_SdIsBusy);
}